#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import sphinx_rtd_theme

#
# The Mesa 3D Graphics Library documentation build configuration file, created by
# sphinx-quickstart on Wed Mar 29 14:08:51 2017.
#
# This file is execfile()d with the current directory set to its
# containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import sys

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
sys.path.append(os.path.abspath('_exts'))


# -- General configuration ------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = ['sphinx.ext.graphviz', 'breathe', 'formatting', 'nir', 'redirects']

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
# source_suffix = ['.rst', '.md']
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = 'The Mesa 3D Graphics Library'
copyright = '1995-2018, Brian Paul'
author = 'Brian Paul'
html_show_copyright = False

html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
version = 'latest'
# The full version, including alpha/beta/rc tags.
release = 'latest'

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = None

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This patterns also effect to html_static_path and html_extra_path
exclude_patterns = []

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = False

# Disable highlighting unless a language is specified, otherwise we'll get
# python keywords highlit in literal blocks.
highlight_language = "none"


# -- Options for HTML output ----------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'sphinx_rtd_theme'

html_favicon = "favicon.ico"

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
#
html_theme_options = {
  'display_version': False,
}

html_context = {
  'display_gitlab': True,
  'gitlab_host': 'gitlab.freedesktop.org',
  'gitlab_user': 'mesa',
  'gitlab_repo': 'mesa',
  'gitlab_version': 'main',
  'conf_py_path': '/docs/',
}

html_copy_source = False

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = []

html_extra_path = [
  '_extra/',
  'release-maintainers-keys.asc',
  'features.txt',
  'libGL.txt',
  'README.UVD',
  'README.VCE',
]

html_redirects = [
  ('drivers/vmware-guest', 'drivers/svga3d.html'),
  ('gallium/drivers/freedreno', 'drivers/freedreno.html'),
  ('gallium/drivers/freedreno/ir3-notes', 'drivers/freedreno/ir3-notes.html'),
  ('gallium/drivers/llvmpipe', 'drivers/llvmpipe.html'),
  ('gallium/drivers/openswr', 'drivers/openswr.html'),
  ('gallium/drivers/openswr/faq', 'drivers/openswr/faq.html'),
  ('gallium/drivers/openswr/knobs', 'drivers/openswr/knobs.html'),
  ('gallium/drivers/openswr/profiling', 'drivers/openswr/profiling.html'),
  ('gallium/drivers/openswr/usage', 'drivers/openswr/usage.html'),
  ('gallium/drivers/zink', 'drivers/zink.html'),
  ('llvmpipe', 'drivers/llvmpipe.html'),
  ('postprocess', 'gallium/postprocess.html'),
  ('versions', 'relnotes.html'),
  ('vmware-guest', 'drivers/vmware-guest.html'),
  ('webmaster', 'https://www.mesa3d.org/website/'),
]


# -- Options for HTMLHelp output ------------------------------------------

# Output file base name for HTML help builder.
htmlhelp_basename = 'TheMesa3DGraphicsLibrarydoc'


# -- Options for LaTeX output ---------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    #
    # 'papersize': 'letterpaper',

    # The font size ('10pt', '11pt' or '12pt').
    #
    # 'pointsize': '10pt',

    # Additional stuff for the LaTeX preamble.
    #
    # 'preamble': '',

    # Latex figure (float) alignment
    #
    # 'figure_align': 'htbp',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    (master_doc, 'TheMesa3DGraphicsLibrary.tex', 'The Mesa 3D Graphics Library Documentation',
     'Brian Paul', 'manual'),
]


# -- Options for manual page output ---------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (master_doc, 'themesa3dgraphicslibrary', 'The Mesa 3D Graphics Library Documentation',
     [author], 1)
]


# -- Options for Texinfo output -------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (master_doc, 'TheMesa3DGraphicsLibrary', 'The Mesa 3D Graphics Library Documentation',
     author, 'TheMesa3DGraphicsLibrary', 'One line description of project.',
     'Miscellaneous'),
]

# -- Options for Graphviz -------------------------------------------------

graphviz_output_format = 'svg'

# -- Options for breathe --------------------------------------------------
breathe_projects = {
    'mesa' : 'doxygen_xml',
}
breathe_default_project = 'mesa'
breathe_show_define_initializer = True
breathe_show_enumvalue_initializer = True
